package com.fzm.common.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fzm.common.constant.SystemConstant;
import com.fzm.common.entity.*;
import com.fzm.common.entity.dto.CopyrightDTO;
import com.fzm.common.entity.vo.CopyrightVo;
import com.fzm.common.enums.CopyrightApplyStateEnum;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.mapper.CopyrightApplyMapper;
import com.fzm.common.service.*;
import com.fzm.common.utils.JwtUtil;
import com.fzm.common.utils.OssUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @author tangtuo
 * @date 2021/12/9 11:40
 */
@Service
@Transactional(rollbackFor = RuntimeException.class)
public class CopyrightApplyServiceImpl extends ServiceImpl<CopyrightApplyMapper, CopyrightApply> implements CopyrightApplyService {

    @Resource
    private CopyrightApplyMapper copyrightApplyMapper;

    @Resource
    private CopyrightFileService copyrightFileService;

    @Resource
    private CopyrightApplyOwnerRelationService copyrightApplyOwnerRelationService;

    @Resource
    private CopyrightAuthorService copyrightAuthorService;

    @Resource
    private CopyrightAuthorityRelationService copyrightAuthorityRelationService;

    @Resource
    private AuthorityService authorityService;

    @Resource
    private CopyrightOwnerService copyrightOwnerService;

    @Resource
    private UserService userService;

    @Resource
    private NftService nftService;

    @Resource
    private OssUtil ossUtil;

    @Override
    public Integer submit(CopyrightDTO copyrightDTO) {
        // 判断此nft是否属于当前登录用户所有
        Nft nft = nftService.getByNftHash(copyrightDTO.getNftHash());
        User user = userService.getUserByToken();
        if (nft == null || !nft.getUserId().equals(user.getId())) {
            throw GlobalException.newException(ResultCode.FAILED, "请先确认此nft的有效性");
        }
        // 如果发表状态是已发表的话,需要传入首次发表日期和省市
        if (SystemConstant.BOOLEAN_DATA_TRUE.equals(copyrightDTO.getPublishState()) &&
                (copyrightDTO.getFirstPublishDate() == null ||
                        StringUtils.isAnyBlank(copyrightDTO.getFirstPublishCity(), copyrightDTO.getFirstPublishProvince()))) {
            throw GlobalException.newException(ResultCode.VALIDATE_FAILED);
        }
        CopyrightApply copyrightApply = new CopyrightApply(copyrightDTO);
        // 首次提交,登记状态为待提交
        copyrightApply.setRegisterState(CopyrightApplyStateEnum.TO_BE_REVIEWED.getCode());
        copyrightApply.setUserId(user.getId());
        save(copyrightApply);
        // 保存附件
        ArrayList<CopyrightFile> files = new ArrayList<>();
        for (CopyrightDTO.File file : copyrightDTO.getFiles()) {
            CopyrightFile copyrightFile = new CopyrightFile(file);
            copyrightFile.setNftHash(copyrightApply.getNftHash());
            copyrightFile.setCopyrightId(copyrightApply.getId());
            files.add(copyrightFile);
        }
        copyrightFileService.saveBatch(files);
        // 保存著作权人信息
        ArrayList<CopyrightApplyOwnerRelation> owners = new ArrayList<>();
        for (Integer ownerId : copyrightDTO.getOwnerIds()) {
            owners.add(new CopyrightApplyOwnerRelation(copyrightApply.getId(), ownerId));
        }
        copyrightApplyOwnerRelationService.saveBatch(owners);
        // 保存作者信息
        ArrayList<CopyrightAuthor> authors = new ArrayList<>();
        for (CopyrightDTO.Author author : copyrightDTO.getAuthors()) {
            CopyrightAuthor copyrightAuthor = new CopyrightAuthor(author);
            copyrightAuthor.setCopyrightId(copyrightApply.getId());
            authors.add(copyrightAuthor);
        }
        copyrightAuthorService.saveBatch(authors);
        // 保存权力信息
        ArrayList<CopyrightAuthorityRelation> relations = new ArrayList<>();
        for (Integer authorityId : copyrightDTO.getAuthorityIds()) {
            CopyrightAuthorityRelation relation = new CopyrightAuthorityRelation(copyrightApply.getId(), authorityId);
            relations.add(relation);
        }
        copyrightAuthorityRelationService.saveBatch(relations);
        return copyrightApply.getId();
    }

    @Override
    public List<CopyrightApply> list(Integer userId, Integer state) {
        return copyrightApplyMapper.list(userId, state);
    }

    @Override
    public Boolean reject(Integer id) {
        CopyrightApply copyright = getById(id);
        if (copyright == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "此版权登记记录不存在,请核对后重试");
        }
        if (CopyrightApplyStateEnum.TO_BE_REVIEWED.getCode() != copyright.getRegisterState()) {
            throw GlobalException.newException(ResultCode.OPERATION_FAILED, "此版权申请记录已提交,无法驳回");
        }
        copyright.setRegisterState(CopyrightApplyStateEnum.REJECTED.getCode());
        return updateById(copyright);
    }


    @Override
    public Boolean delete(Integer id) {
        CopyrightApply copyright = getById(id);
        if (copyright == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "此版权登记记录不存在,请核对后重试");
        }
        // 只有驳回和登记失败的才能删除
        if (CopyrightApplyStateEnum.REJECTED.getCode() != copyright.getRegisterState() &&
                CopyrightApplyStateEnum.FAILED.getCode() != copyright.getRegisterState()) {
            throw GlobalException.newException(ResultCode.OPERATION_FAILED, "只有驳回或登记失败的版权记录才能删除");
        }
        // 删除权力取得方式的附件
        if (StringUtils.isNotBlank(copyright.getAuthorityAcquireMode())) {
            ossUtil.delete(copyright.getAuthorityAcquireMode());
        }
        // 删除权力归属证明附件
        if (StringUtils.isNotBlank(copyright.getAuthorityAscriptionMode())) {
            ossUtil.delete(copyright.getAuthorityAcquireMode());
        }
        // 删除附件列表
        copyrightFileService.delByCopyrightId(id);
        // 删除作者列表
        copyrightAuthorService.delByCopyrightId(id);
        // 删除版权权力拥有情况
        copyrightAuthorityRelationService.delByCopyrightId(id);
        return copyrightApplyMapper.deleteById(id) > 0;
    }

    @Override
    public CopyrightVo getDetail(Integer id) {
        CopyrightApply copyrightApply = getById(id);
        CopyrightVo copyrightVo = new CopyrightVo(copyrightApply);
        // 查询附件列表
        List<CopyrightFile> files = copyrightFileService.getByCopyrightId(id);
        copyrightVo.setFiles(files);
        // 查询作者列表
        List<CopyrightAuthor> authors = copyrightAuthorService.getByCopyrightId(id);
        copyrightVo.setAuthors(authors);
        // 查询著作权人列表
        List<CopyrightOwner> owners = copyrightOwnerService.getByCopyrightId(id);
        // 查询权力列表
        List<Authority> authorities = authorityService.getByCopyrightId(id);
        copyrightVo.setAuthorities(authorities);
        return copyrightVo;
    }
}
